use std::sync::mpsc;

use egui::{
    Align, CentralPanel, Context, FontFamily, Label, RichText, ScrollArea, ViewportBuilder,
    ViewportId,
};

use crate::emulator::{EmulatorClient, EmulatorCommand, SimId};

use super::AppWindow;

pub struct TerminalWindow {
    sim_id: SimId,
    receiver: mpsc::Receiver<String>,
    text: String,
}

impl TerminalWindow {
    pub fn new(sim_id: SimId, client: &EmulatorClient) -> Self {
        let (sender, receiver) = mpsc::channel();
        client.send_command(EmulatorCommand::WatchStdout(sim_id, sender));
        Self {
            sim_id,
            receiver,
            text: String::new(),
        }
    }
}

impl AppWindow for TerminalWindow {
    fn viewport_id(&self) -> ViewportId {
        ViewportId::from_hash_of(format!("terminal-{}", self.sim_id))
    }

    fn sim_id(&self) -> SimId {
        self.sim_id
    }

    fn initial_viewport(&self) -> ViewportBuilder {
        ViewportBuilder::default()
            .with_title(format!("Terminal ({})", self.sim_id))
            .with_inner_size((640.0, 480.0))
    }

    fn show(&mut self, ctx: &Context) {
        if let Ok(text) = self.receiver.try_recv() {
            self.text.push_str(&text);
        }
        CentralPanel::default().show(ctx, |ui| {
            ScrollArea::vertical()
                .stick_to_bottom(true)
                .auto_shrink([false, false])
                .show(ui, |ui| {
                    let label = Label::new(RichText::new(&self.text).family(FontFamily::Monospace))
                        .halign(Align::LEFT)
                        .wrap();
                    ui.add(label);
                });
        });
    }
}
